from eqsig.single import Signal, AccSignal, significant_range
from eqsig import design_spectra
from eqsig.functions import *
from eqsig.multiple import Cluster, combine_at_angle, compute_rotated
from eqsig.measures import significant_duration, calculate_peak
